diff --exclude=CVS -ur u-boot-cvs/drivers/lan91c96.c u-boot/drivers/lan91c96.c
--- u-boot-cvs/drivers/lan91c96.c	2003-06-27 17:32:37.000000000 -0400
+++ u-boot/drivers/lan91c96.c	2004-02-20 17:48:56.000000000 -0500
@@ -62,6 +62,10 @@
 #include <command.h>
 #include "lan91c96.h"
 #include <net.h>
+#ifdef __LITTLE_ENDIAN	
+#include <linux/byteorder/swab.h>
+#endif
+
 
 #ifdef CONFIG_DRIVER_LAN91C96
 
@@ -83,24 +87,25 @@
 */
 #define MEMORY_WAIT_TIME 16
 
-#define SMC_DEBUG 0
-
+/*
+ * SMC_DEBUG comes from the header file...
+ */
 #if (SMC_DEBUG > 2 )
-#define PRINTK3(args...) printf(args)
+ #define PRINTK3(args...) printf(args)
 #else
-#define PRINTK3(args...)
+ #define PRINTK3(args...)
 #endif
 
 #if SMC_DEBUG > 1
-#define PRINTK2(args...) printf(args)
+ #define PRINTK2(args...) printf(args)
 #else
-#define PRINTK2(args...)
+ #define PRINTK2(args...)
 #endif
 
 #ifdef SMC_DEBUG
-#define PRINTK(args...) printf(args)
+ #define PRINTK(args...) printf(args)
 #else
-#define PRINTK(args...)
+ #define PRINTK(args...)
 #endif
 
 
@@ -112,20 +117,20 @@
  *
  *------------------------------------------------------------------------
  */
-#define CARDNAME "LAN91C96"
+#define CARDNAME 		"LAN91C96"
 
-#define SMC_BASE_ADDRESS CONFIG_LAN91C96_BASE
+#define SMC_BASE_ADDRESS 	CONFIG_LAN91C96_BASE
 
-#define SMC_DEV_NAME "LAN91C96"
-#define SMC_ALLOC_MAX_TRY 5
-#define SMC_TX_TIMEOUT 30
+#define SMC_DEV_NAME 		"LAN91C96"
+#define SMC_ALLOC_MAX_TRY 	5
+#define SMC_TX_TIMEOUT 		30
 
-#define ETH_ZLEN 60
+#define ETH_ZLEN 		60
 
 #ifdef  CONFIG_LAN91C96_USE_32_BIT
-#define USE_32_BIT  1
+ #define USE_32_BIT  		1
 #else
-#undef USE_32_BIT
+ #undef USE_32_BIT
 #endif
 
 /*-----------------------------------------------------------------
@@ -178,6 +183,18 @@
  */
 static int smc_rcv (void);
 
+/*
+ * some convenient interfaces to the I/O and Attribute spaces
+ */
+static volatile unsigned *ioaddr = (unsigned *)CONFIG_LAN91C96_BASE;
+static volatile unsigned *attaddr = (unsigned *)CONFIG_LAN91C96_ATTR;
+
+#if SMC_DEBUG > 2
+/*
+ * A rather simple routine to print out a packet for debugging purposes.
+ */
+static void print_packet (byte *, int);
+#endif
 
 /* ------------------------------------------------------------
  * Internal routines
@@ -200,16 +217,16 @@
 	}
 }
 
+
+#if 0
 /*
  * smc_get_macaddr is no longer used. If you want to override the default
  * mac address, call smc_get_mac_addr as a part of the board initialisation.
  */
-
-#if 0
 void smc_get_macaddr (byte * addr)
 {
 	/* MAC ADDRESS AT FLASHBLOCK 1 / OFFSET 0x10 */
-	unsigned char *dnp1110_mac = (unsigned char *) (0xE8000000 + 0x20010);
+	unsigned char *dnp1110_mac = (unsigned char *) (CONFIG_LAN91C96_ATTR + 0x20010);
 	int i;
 
 
@@ -236,17 +253,11 @@
 
 	SMC_SELECT_BANK (0);
 	mem_info = SMC_inw (LAN91C96_MIR);
-	PRINTK2 ("Memory: %4d available\n", (mem_info >> 8) * 2048);
+	PRINTK2("LAN91C96 Memory: %4d available\n", (mem_info >> 8) * 2048);
 
 	SMC_SELECT_BANK (old_bank);
 }
 
-/*
- * A rather simple routine to print out a packet for debugging purposes.
- */
-#if SMC_DEBUG > 2
-static void print_packet (byte *, int);
-#endif
 
 /* #define tx_done(dev) 1 */
 
@@ -263,14 +274,25 @@
 
 static int poll4int (byte mask, int timeout)
 {
-	int tmo = get_timer (0) + timeout * CFG_HZ;
-	int is_timeout = 0;
-	word old_bank = SMC_inw (LAN91C96_BANK_SELECT);
-
-	PRINTK2 ("Polling...\n");
-	SMC_SELECT_BANK (2);
-	while ((SMC_inw (LAN91C96_INT_STATS) & mask) == 0) {
-		if (get_timer (0) >= tmo) {
+	int tmo;
+	int is_timeout;
+	word old_bank, tmp, last;
+
+	PRINTK("LAN91C96 IRQ Polling...\n");
+	
+	tmo = get_timer(0) + timeout * CFG_HZ;
+	is_timeout = 0;
+	old_bank = ioaddr[ LAN91C96_BANK_SELECT ];
+	
+	SMC_SELECT_BANK(2);
+	last = 0;
+	while (((tmp = ioaddr[ LAN91C96_INT_STATS ]) & mask) == 0) {
+		if (last != tmp)
+		{
+			PRINTK("\t: mask=0x%x, status: 0x%x\n", mask, tmp);
+			last = tmp;
+		}
+		if (get_timer(0) >= tmo) {
 			is_timeout = 1;
 			break;
 		}
@@ -336,22 +358,55 @@
 */
 static void smc_enable ()
 {
-	PRINTK2 ("%s:smc_enable\n", SMC_DEV_NAME);
-	SMC_SELECT_BANK (0);
+	unsigned short tmp;
 
-	/* Initialize the Memory Configuration Register. See page
-	   49 of the LAN91C96 data sheet for details. */
-	SMC_outw (LAN91C96_MCR_TRANSMIT_PAGES, LAN91C96_MCR);
-
-	/* Initialize the Transmit Control Register */
-	SMC_outw (LAN91C96_TCR_TXENA, LAN91C96_TCR);
-	/* Initialize the Receive Control Register
-	 * FIXME:
-	 * The promiscuous bit set because I could not receive ARP reply
-	 * packets from the server when I send a ARP request. It only works
-	 * when I set the promiscuous bit
+	/* get current config */
+	tmp = ioaddr[ LAN91C96_CONFIG ];
+	PRINTK("SMsC config::: 0x%x\n", tmp );
+	
+	/* Jump to bank 3 and read IOS */
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK3;
+	tmp = ioaddr[ LAN91C96_MGMT ];
+	PRINTK("SMsC switches: 0x%x\n", tmp );
+	tmp = ioaddr[ LAN91C96_REVISION ];
+	PRINTK("SMsC revision: 0x%x\n", tmp & 0xf0);
+	PRINTK("SMsC Chip Id:: 0x%x\n", tmp & 0x0f);
+	udelay( 750 );
+	
+	/* 
+	 * set the Squelch bit... reduce the req'd valid signal from 300mV
+	 * to a mere 180mV.  helps with some network hardware issues.
 	 */
-	SMC_outw (LAN91C96_RCR_RXEN | LAN91C96_RCR_PRMS, LAN91C96_RCR);
+	 
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK1;
+	tmp = ioaddr[ LAN91C96_CONFIG ];
+	tmp |= LAN91C96_CR_SET_SQLCH | LAN91C96_CR_NO_WAIT | LAN91C96_CR_16BIT;
+	tmp &= ~(LAN91C96_CR_DIS_LINK | LAN91C96_CR_AUI_SELECT);
+	ioaddr[ LAN91C96_CONFIG ] = tmp;
+	
+	/* setup control: TxEnable + AutoRelease */
+	
+	tmp = LAN91C96_CTR_TE_ENABLE | LAN91C96_CTR_BIT_8;  /* if you want to skip TX-complete Int's and releases, set this and fixup code: | LAN91C96_CTR_AUTO_RELEASE */
+	ioaddr[ LAN91C96_CONTROL ] = tmp;
+	
+	/* go to bank zero.  reserve a minimum 1500-byte packet space (6 pages),
+	 * then enable Tx and zero-pad under-sized packets (min 64byte pkt) */
+	
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK0;
+	ioaddr[ LAN91C96_MCR ] = LAN91C96_MCR_TRANSMIT_PAGES;
+	ioaddr[ LAN91C96_TCR ] = LAN91C96_TCR_TXENA | LAN91C96_TCR_PAD_EN | LAN91C96_TCR_FDSE;
+	
+	/* go disable any interrupts, and then enable Rx, and wait for settling */
+	
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK2;
+	ioaddr[ LAN91C96_INT_MASK ] = 0;
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK0;
+	
+	/* THIS MUST BE LAST ! */
+	ioaddr[ LAN91C96_RCR ] = LAN91C96_RCR_RXEN | LAN91C96_RCR_PRMS;	
+	udelay( 750 );
+	
+	PRINTK("LAN91C96 initialized and enabled!\n");
 }
 
 /*
@@ -370,7 +425,7 @@
  */
 static void smc_shutdown ()
 {
-	PRINTK2 (CARDNAME ":smc_shutdown\n");
+	PRINTK(CARDNAME ":smc_shutdown\n");
 
 	/* no more interrupts for me */
 	SMC_SELECT_BANK (2);
@@ -380,6 +435,8 @@
 	SMC_SELECT_BANK (0);
 	SMC_outb (0, LAN91C96_RCR);
 	SMC_outb (0, LAN91C96_TCR);
+
+	PRINTK("smc-shutdown(): done\n");
 }
 
 
@@ -413,7 +470,7 @@
 	byte status;
 
 
-	PRINTK3 ("%s:smc_hardware_send_packet\n", SMC_DEV_NAME);
+	PRINTK("%s:smc_hardware_send_packet ,len=0x%x\n", SMC_DEV_NAME, packet_length);
 
 	length = ETH_ZLEN < packet_length ? packet_length : ETH_ZLEN;
 
@@ -490,8 +547,8 @@
 			 SMC_DEV_NAME, length);
 
 #if SMC_DEBUG > 2
-	printf ("Transmitting Packet\n");
-	print_packet (buf, length);
+	printf("Transmitting Packet\n");
+	print_packet(buf, length);
 #endif
 
 	/* send the packet length ( +6 for status, length and ctl byte )
@@ -592,7 +649,7 @@
 {
 	int i;			/* used to set hw ethernet address */
 
-	PRINTK2 ("%s:smc_open\n", SMC_DEV_NAME);
+	PRINTK("%s:smc_open\n", SMC_DEV_NAME);
 
 	/* reset the hardware */
 
@@ -601,13 +658,19 @@
 
 	SMC_SELECT_BANK (1);
 
+	PRINTK("Setting SMC EthAddr\n");
 	for (i = 0; i < 6; i += 2) {
 		word address;
 
 		address = smc_mac_addr[i + 1] << 8;
 		address |= smc_mac_addr[i];
+#ifdef __LITTLE_ENDIAN	
+		address = __swab16( address );
+#endif
+		PRINTK2("\tByte: 0x%02x\n", address);
 		SMC_outw (address, LAN91C96_IA0 + i);
 	}
+	PRINTK("smc_open():done\n");
 	return 0;
 }
 
@@ -634,7 +697,6 @@
 	dword stat_len;
 #endif
 
-
 	SMC_SELECT_BANK (2);
 	packet_number = SMC_inw (LAN91C96_FIFO);
 
@@ -659,7 +721,7 @@
 
 	packet_length &= 0x07ff;	/* mask off top bits */
 
-	PRINTK2 ("RCV: STATUS %4x LENGTH %4x\n", status, packet_length);
+	PRINTK2("RCV: STATUS %4x LENGTH %4x\n", status, packet_length);
 
 	if (!(status & FRAME_FILTER)) {
 		/* Adjust for having already read the first two words */
@@ -782,7 +844,9 @@
 
 int eth_init (bd_t * bd)
 {
+	PRINTK("LAN91C96: eth_init()\n");
 	smc_open ();
+	PRINTK("LAN91C96: eth_init():done\n");
 	return 0;
 }
 
@@ -823,30 +887,41 @@
  */
 static int smc_hw_init ()
 {
-	unsigned short status_test;
+	unsigned short tmp;
 
-	/* The attribute register of the LAN91C96 is located at address
-	   0x0e000000 on the lubbock platform */
-	volatile unsigned *attaddr = (unsigned *) (0x0e000000);
+	PRINTK("smc_hw_init()\n");
+	PRINTK("LAN91C96: Attribute space is set to 0x%08lx\n", attaddr);
 
+	tmp = attaddr[LAN91C96_ECOR];
+	
+	PRINTK("\tECOR is currently 0x%x\n", tmp );
+	
 	/* first reset, then enable the device. Sequence is critical */
 	attaddr[LAN91C96_ECOR] |= LAN91C96_ECOR_SRESET;
-	udelay (100);
+	udelay( 750 );
 	attaddr[LAN91C96_ECOR] &= ~LAN91C96_ECOR_SRESET;
+	udelay( 750 );
 	attaddr[LAN91C96_ECOR] |= LAN91C96_ECOR_ENABLE;
+	udelay( 750 );
 
 	/* force 16-bit mode */
 	attaddr[LAN91C96_ECSR] &= ~LAN91C96_ECSR_IOIS8;
-	udelay (100);
+	udelay( 750 );
+	
+	/* access I/O space to select bank 1 */
+	ioaddr[ LAN91C96_BANK_SELECT ] = BANK1;
 
 	/* check if the I/O address is correct, the upper byte of the
 	   bank select register should read 0x33 */
 
-	status_test = SMC_inw (LAN91C96_BANK_SELECT);
-	if ((status_test & 0xFF00) != 0x3300) {
+	tmp = ioaddr[ LAN91C96_BANK_SELECT ] ;
+	if ((tmp & 0xFF00) != 0x3300) {
 		printf ("Failed to initialize ethernetchip\n");
 		return 1;
 	}
+	PRINTK("Bank Sel. Status is now 0x%x\n", tmp );
+	
+	PRINTK("smc_hw_init():done\n");
 	return 0;
 }
 
